"""
:mod:`zsl_jwt.module`
---------------------
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from builtins import *  # NOQA

from flask.config import Config
from injector import Module
from injector import provides
from zsl import inject

from zsl_jwt.configuration import JWTConfiguration


class JWTModule(Module):
    """
    JWT module activation. Add to your IoCContainer to enable JWT support.

    It just provides the :class:`zsl_jwt.configuration.JWTConfiguration` so
    that the users may read the JWT configuration.
    """

    #: Variable name in config used for JWT configuration
    JWT_CONFIG_NAME = 'JWT'

    @provides(JWTConfiguration)
    @inject(config=Config)
    def provide_jwt_configuration(self, config):
        # type: (Config) -> JWTConfiguration
        """
        Returns the JWT configuration.

        :param config: Injected. Configuration object
        :return: Current JWT configuration.
        """
        return config.get(JWTModule.JWT_CONFIG_NAME)



from datetime import timedelta
from unittest.case import TestCase

from zsl.application.containers.container import IoCContainer
from zsl.application.modules.context_module import DefaultContextModule
from zsl.application.modules.logger_module import LoggerModule
from zsl.testing.zsl import ZslTestCase
from zsl.testing.zsl import ZslTestConfiguration

from zsl_jwt.codec import ZslJwtExpiredSignatureError
from zsl_jwt.codec import ZslJwtInvalidAudienceError
from zsl_jwt.codec import ZslJwtInvalidNbfClaimError
from zsl_jwt.codec import decode
from zsl_jwt.codec import encode
from zsl_jwt.configuration import JWTConfiguration
from zsl_jwt.configuration import JWTProfile
from zsl_jwt.module import JWTModule


class ZslJwtContainer(IoCContainer):
    logger = LoggerModule
    context = DefaultContextModule
    jwt_module = JWTModule


class TestEncode(ZslTestCase, TestCase):
    SECRET = 'secret'
    PAYLOAD = {'a': 'b', 'c': {'d': 'e'}}
    ZSL_TEST_CONFIGURATION = ZslTestConfiguration(
        app_name='zsl_jwt_encode_test',
        container=ZslJwtContainer,
        config_object={
            JWTModule.JWT_CONFIG_NAME: JWTConfiguration(
                default_secret=SECRET,
                profiles={
                    'fail_nbf_claim': JWTProfile(
                        SECRET,
                        not_before=timedelta(seconds=30),
                        audience='fail_nbf_claim'
                    ),
                    'fail_exp_claim': JWTProfile(
                        SECRET,
                        expiration=timedelta(seconds=-30),
                        audience='fail_exp_claim'
                    ),
                    'correct': JWTProfile(
                        SECRET,
                        audience='correct'
                    ),
                }
            )
        }
    )

    def testEncodeDefaultProfile(self):
        token = encode(TestEncode.PAYLOAD)
        decoded_payload = decode(token)
        self.assertEqual(TestEncode.PAYLOAD, decoded_payload,
                         "The encoding/decoding process must result the "
                         "original payload.")

    def testEncodeInvalidAudienceClaim(self):
        token = encode(TestEncode.PAYLOAD)
        with self.assertRaises(ZslJwtInvalidAudienceError):
            decode(token, profile='correct')

    def testEncodeInvalidNotBeforeClaim(self):
        token = encode(TestEncode.PAYLOAD, profile='fail_nbf_claim')
        with self.assertRaises(ZslJwtInvalidNbfClaimError):
            decode(token, profile='fail_nbf_claim')

    def testEncodeExpiredSignatureClaim(self):
        token = encode(TestEncode.PAYLOAD, profile='fail_exp_claim')
        with self.assertRaises(ZslJwtExpiredSignatureError):
            decode(token, profile='fail_exp_claim')
